﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Optimization;
using HeuristicLab.Data;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Common;
using HeuristicLab.Common.Resources;
using HeuristicLab.Analysis.FitnessLandscape.DataTables;
using System.Drawing;
using HeuristicLab.Analysis.FitnessLandscape.Analysis;

namespace HeuristicLab.Analysis.FitnessLandscape {

  [Item("Information Stability Aggregator", "Aggregates information stability analyses.")]
  [StorableClass]
  public class InformationStabilityAggregator : Aggregator<InformationStabilityTable> {

    [StorableConstructor]
    protected InformationStabilityAggregator(bool deserializing) : base(deserializing) { }
    protected InformationStabilityAggregator(InformationStabilityAggregator original, Cloner cloner) : base(original, cloner) { }
    public InformationStabilityAggregator() { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new InformationStabilityAggregator(this, cloner);
    }

    public override IResult CreateResult() {
      return new Result("Information Stabilities Summary", Aggregate(items));
    }

    public static DataTable Aggregate(List<InformationStabilityTable> informationStabilityTables) {
      DataTable table = new DataTable("Information Stabilities");
      if (informationStabilityTables.Count == 0)
        return table;
      try {
        List<double> values = new List<double>();
        foreach (var t in informationStabilityTables) {
          if (t.Rows.Count > 0 && t.Rows.First().Values.Count > 0)
            values.Add(t.Rows.First().Values.Last());
        }
        DataRow valuesRow = new DataRow("values", "information stability values", values);
        valuesRow.VisualProperties.ChartType = DataRowVisualProperties.DataRowChartType.Points;
        table.Rows.Add(valuesRow);
        double max = values.Max();
        double min = values.Min();
        double avg = values.Average();
        table.Rows.Add(new DataRow("max", "Maximum", values.Select(v => max)));
        table.Rows.Add(new DataRow("min", "Minimum", values.Select(v => min)));
        table.Rows.Add(new DataRow("avg", "Average", values.Select(v => avg)));
      } catch (Exception) { }
      return table;
    }
  }
}
